import { Exception } from "./Exception";
interface Log {
    date: Date;
    result: "OK" | "WARN" | "ERROR";
    elapsedTime: number;
    context: {
        [key: string]: string;
    };
    info: {
        [key: string]: string | undefined;
    };
    action?: string;
    errorCode?: string;
    errorMessage?: string;
}
interface ErrorLogEntry {
    action: string;
    elapsedTime: number;
    errorCode: string;
    errorMessage: string;
    info: {
        [key: string]: string | undefined;
    };
}
/**
 * If eventLogger config is provided in non-DEV environment.
 * All collected logs will automatically sent to {serverURL} in a regular basis.
 *
 * The request will be PUT to the server in the following format:
 *      {events: Log[]}
 */
export interface LoggerConfig {
    serverURL: string;
    maskedKeywords?: RegExp[];
}
export interface Logger {
    addContext(context: {
        [key: string]: string | (() => string);
    }): void;
    /**
     * Add a log item, whose result is OK
     */
    info(action: string, info: {
        [key: string]: string | undefined;
    }, elapsedTime?: number): void;
    /**
     * Add a log item, whose result is WARN
     * @errorCode: Naming in upper-case and underscore, e.g: SOME_DATA
     */
    warn(data: ErrorLogEntry): void;
    /**
     * Add a log item, whose result is ERROR
     * @errorCode: Naming in upper-case and underscore, e.g: SOME_DATA
     */
    error(data: ErrorLogEntry): void;
}
export declare class LoggerImpl implements Logger {
    private environmentContext;
    private logQueue;
    constructor();
    addContext(context: {
        [key: string]: string | (() => string);
    }): void;
    info(action: string, info: {
        [key: string]: string | undefined;
    }, elapsedTime?: number): void;
    warn(data: ErrorLogEntry): void;
    error(data: ErrorLogEntry): void;
    exception(exception: Exception, extra: {
        [key: string]: string | undefined;
    }, action: string): void;
    collect(): Log[];
    empty(): void;
    appendLog(result: "OK" | "WARN" | "ERROR", data: Pick<Log, "action" | "info" | "errorCode" | "errorMessage" | "elapsedTime">): void;
}
export {};
