import React from "react";
import { AppState } from "react-native";
import { delay, call as rawCall } from "redux-saga/effects";
import { app } from "../app";
import { executeAction } from "../module";
export class ModuleProxy {
    constructor(module, actions) {
        this.module = module;
        this.actions = actions;
    }
    getActions() {
        return this.actions;
    }
    attachLifecycle(ComponentType) {
        var _a;
        const moduleName = this.module.name;
        const lifecycleListener = this.module;
        const actions = this.actions;
        return _a = class extends React.PureComponent {
                constructor(props) {
                    super(props);
                    this.lifecycleSagaTask = null;
                    this.successTickCount = 0;
                    this.mountedTime = Date.now();
                    this.onAppStateChange = (nextAppState) => {
                        const { appState } = this.state;
                        if (["inactive", "background"].includes(appState) && nextAppState === "active") {
                            if (lifecycleListener.onAppActive.isLifecycle) {
                                app.store.dispatch(actions.onAppActive());
                            }
                        }
                        else if (appState === "active" && ["inactive", "background"].includes(nextAppState)) {
                            if (lifecycleListener.onAppInactive.isLifecycle) {
                                app.store.dispatch(actions.onAppInactive());
                            }
                        }
                        this.setState({ appState: nextAppState });
                    };
                    this.state = { appState: AppState.currentState };
                }
                componentDidMount() {
                    this.lifecycleSagaTask = app.sagaMiddleware.run(this.lifecycleSaga.bind(this));
                    // According to the document, this API may change soon
                    // Ref: https://facebook.github.io/react-native/docs/appstate#addeventlistener
                    AppState.addEventListener("change", this.onAppStateChange);
                    const props = this.props;
                    if ("navigation" in props) {
                        const navigation = props.navigation;
                        this.focusSubscription = navigation.addListener("didFocus", () => {
                            if (lifecycleListener.onFocus.isLifecycle) {
                                app.store.dispatch(actions.onFocus());
                            }
                        });
                        this.blurSubscription = navigation.addListener("willBlur", () => {
                            if (lifecycleListener.onBlur.isLifecycle) {
                                app.store.dispatch(actions.onBlur());
                            }
                        });
                    }
                }
                componentWillUnmount() {
                    var _a, _b, _c;
                    if (lifecycleListener.onDestroy.isLifecycle) {
                        app.store.dispatch(actions.onDestroy());
                    }
                    app.logger.info(`${moduleName}/@@DESTROY`, {
                        successTickCount: this.successTickCount.toString(),
                        stayingSecond: ((Date.now() - this.mountedTime) / 1000).toFixed(2),
                    });
                    (_a = this.lifecycleSagaTask) === null || _a === void 0 ? void 0 : _a.cancel();
                    (_b = this.blurSubscription) === null || _b === void 0 ? void 0 : _b.remove();
                    (_c = this.focusSubscription) === null || _c === void 0 ? void 0 : _c.remove();
                    AppState.removeEventListener("change", this.onAppStateChange);
                }
                render() {
                    return <ComponentType {...this.props}/>;
                }
                *lifecycleSaga() {
                    var _a;
                    /**
                     * CAVEAT:
                     * Do not use <yield* executeAction> for lifecycle actions.
                     * It will lead to cancellation issue, which cannot stop the lifecycleSaga as expected.
                     *
                     * https://github.com/redux-saga/redux-saga/issues/1986
                     */
                    const props = this.props;
                    const enterActionName = `${moduleName}/@@ENTER`;
                    if (lifecycleListener.onEnter.isLifecycle) {
                        const startTime = Date.now();
                        if ("navigation" in props) {
                            /**
                             * For react navigation < 5, use props.navigation.state.params.
                             * For react navigation 5, use props.route.params.
                             */
                            let routeParams;
                            if (typeof props.navigation.state === "object") {
                                routeParams = props.navigation.state.params;
                            }
                            else {
                                routeParams = (_a = props.route) === null || _a === void 0 ? void 0 : _a.params;
                            }
                            yield rawCall(executeAction, enterActionName, lifecycleListener.onEnter.bind(lifecycleListener), routeParams || {});
                        }
                        else {
                            yield rawCall(executeAction, enterActionName, lifecycleListener.onEnter.bind(lifecycleListener), {});
                        }
                        app.logger.info(enterActionName, { componentProps: JSON.stringify(props) }, Date.now() - startTime);
                    }
                    else {
                        app.logger.info(enterActionName, { componentProps: JSON.stringify(props) });
                    }
                    if (lifecycleListener.onTick.isLifecycle) {
                        const tickIntervalInMillisecond = (lifecycleListener.onTick.tickInterval || 5) * 1000;
                        const boundTicker = lifecycleListener.onTick.bind(lifecycleListener);
                        const tickActionName = `${moduleName}/@@TICK`;
                        while (true) {
                            yield rawCall(executeAction, tickActionName, boundTicker);
                            this.successTickCount++;
                            yield delay(tickIntervalInMillisecond);
                        }
                    }
                }
            },
            _a.displayName = `ModuleBoundary(${moduleName})`,
            // Copy static navigation options, important for navigator
            _a.navigationOptions = ComponentType.navigationOptions,
            _a;
    }
}
//# sourceMappingURL=ModuleProxy.js.map