import { Exception, JavaScriptException } from "../Exception";
import { app } from "../app";
import { spawn } from "../typed-saga";
import { sendEventLogs } from "../platform/bootstrap";
export function errorToException(error) {
    if (error instanceof Exception) {
        return error;
    }
    else {
        let message;
        if (!error) {
            message = "[No Message]";
        }
        else if (typeof error === "string") {
            message = error;
        }
        else if (error instanceof Error) {
            message = error.message;
        }
        else {
            try {
                message = JSON.stringify(error);
            }
            catch (e) {
                message = "[Unknown]";
            }
        }
        return new JavaScriptException(message);
    }
}
export function captureError(error, action, extra = {}) {
    if (process.env.NODE_ENV === "development") {
        console.error(`[framework] Error captured from [${action}]`, error);
    }
    const exception = errorToException(error);
    const errorStacktrace = error instanceof Error ? error.stack : undefined;
    const info = Object.assign(Object.assign({}, extra), { stacktrace: errorStacktrace });
    app.logger.exception(exception, info, action);
    app.sagaMiddleware.run(runUserErrorHandler, app.errorHandler, exception);
    return exception;
}
let isUserErrorHandlerRunning = false;
export function* runUserErrorHandler(handler, exception) {
    // For app, report errors to event server ASAP, in case of sudden termination
    yield spawn(sendEventLogs);
    if (isUserErrorHandlerRunning)
        return;
    try {
        isUserErrorHandlerRunning = true;
        yield* handler(exception);
    }
    catch (e) {
        console.warn("[framework] Fail to execute user-defined error handler", e);
    }
    finally {
        isUserErrorHandlerRunning = false;
    }
}
//# sourceMappingURL=error-util.js.map