/**
 * Attention:
 * Do NOT use "axios" for network utils in React Native.
 *
 * Explanation:
 * 1) Axios supports [nodejs-http] and [XHR] only, does not support [fetch] API.
 * 2) Although React Native has a XHR-same interface, but it has a bit difference on CORS policy.
 * 3) If XHR is used in React Native (i.e, what axios does), customized response header cannot be retrieved.
 *
 * Ref:
 * https://stackoverflow.com/questions/37897523/axios-get-access-to-response-header-fields
 */
/// <reference types="react-native" />
declare type RequestHeaderInterceptor = (headers: Headers) => void | Promise<void>;
declare type ResponseHeaderInterceptor = (headers: Headers) => void | Promise<void>;
export declare function setRequestHeaderInterceptor(_: RequestHeaderInterceptor): void;
export declare function setResponseHeaderInterceptor(_: ResponseHeaderInterceptor): void;
export declare function ajax<TRequest, TResponse>(method: string, path: string, pathParams: object, request: TRequest, skipInterceptor?: boolean): Promise<TResponse>;
export declare function uri<Request>(path: string, request: Request): string;
export declare function urlParams(path: string, params: object): string;
export declare function queryString(params: {
    [key: string]: any;
} | null | undefined): string;
export {};
